@@ -26,7 +26,7 @@ int main (int argc, char **argv)
 
 	gtk_init (&argc, &argv);
 
-	dialog = cc_background_chooser_dialog_new ();
+	dialog = cc_background_chooser_dialog_new (NULL);
 	gtk_window_set_modal (GTK_WINDOW (dialog), FALSE);
 	g_signal_connect (G_OBJECT (dialog), "response",
 			  G_CALLBACK (on_dialog_response), NULL);
