@@ -87,8 +87,10 @@ struct _CcPrintersPanelPrivate
   guint            subscription_renewal_id;
   guint            cups_status_check_id;
   guint            dbus_subscription_id;
+  guint            remove_printer_timeout_id;
 
   GtkWidget    *headerbar_buttons;
+  GtkRevealer  *notification;
   PPDList      *all_ppds_list;
   GCancellable *get_all_ppds_cancellable;
   GCancellable *subscription_renew_cancellable;
@@ -98,12 +100,14 @@ struct _CcPrintersPanelPrivate
   gchar    *new_printer_location;
   gchar    *new_printer_make_and_model;
   gboolean  new_printer_on_network;
-  gboolean  select_new_printer;
 
   gchar    *renamed_printer_name;
+  gchar    *deleted_printer_name;
 
   GHashTable *printer_entries;
 
+  GtkSizeGroup *size_group;
+
   gpointer dummy;
 };
 
@@ -176,6 +180,23 @@ cc_printers_panel_constructed (GObject *object)
                             widget);
 }
 
+static void
+printer_removed_cb (GObject      *source_object,
+                    GAsyncResult *result,
+                    gpointer      user_data)
+{
+  GError *error = NULL;
+
+  pp_printer_delete_finish (PP_PRINTER (source_object), result, &error);
+  g_object_unref (source_object);
+
+  if (error != NULL)
+    {
+      g_warning ("Printer could not be deleted: %s", error->message);
+      g_error_free (error);
+    }
+}
+
 static void
 cc_printers_panel_dispose (GObject *object)
 {
@@ -224,6 +245,12 @@ cc_printers_panel_dispose (GObject *object)
       priv->cups_status_check_id = 0;
     }
 
+  if (priv->remove_printer_timeout_id > 0)
+    {
+      g_source_remove (priv->remove_printer_timeout_id);
+      priv->remove_printer_timeout_id = 0;
+    }
+
   if (priv->all_ppds_list)
     {
       ppd_list_free (priv->all_ppds_list);
@@ -237,6 +264,19 @@ cc_printers_panel_dispose (GObject *object)
       priv->get_all_ppds_cancellable = NULL;
     }
 
+  if (priv->deleted_printer_name != NULL)
+    {
+      PpPrinter *printer;
+
+      printer = pp_printer_new (priv->deleted_printer_name);
+      g_clear_pointer (&priv->deleted_printer_name, g_free);
+
+      pp_printer_delete_async (printer,
+                               NULL,
+                               printer_removed_cb,
+                               NULL);
+    }
+
   g_clear_pointer (&priv->printer_entries, g_hash_table_destroy);
 
   G_OBJECT_CLASS (cc_printers_panel_parent_class)->dispose (object);
@@ -588,6 +628,108 @@ free_dests (CcPrintersPanel *self)
   priv->num_dests = 0;
 }
 
+static void
+cancel_notification_timeout (CcPrintersPanel *self)
+{
+  CcPrintersPanelPrivate *priv;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  if (priv->remove_printer_timeout_id == 0)
+    return;
+
+  g_source_remove (priv->remove_printer_timeout_id);
+
+  priv->remove_printer_timeout_id = 0;
+}
+
+static void
+on_printer_deletion_undone (GtkButton *button,
+                            gpointer   user_data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  gtk_revealer_set_reveal_child (priv->notification, FALSE);
+
+  g_clear_pointer (&priv->deleted_printer_name, g_free);
+  actualize_printers_list (self);
+
+  cancel_notification_timeout (self);
+}
+
+static void
+on_notification_dismissed (GtkButton *button,
+                           gpointer   user_data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  if (priv->deleted_printer_name != NULL)
+    {
+      PpPrinter *printer;
+
+      printer = pp_printer_new (priv->deleted_printer_name);
+      pp_printer_delete_async (printer,
+                               NULL,
+                               printer_removed_cb,
+                               NULL);
+
+      g_clear_pointer (&priv->deleted_printer_name, g_free);
+    }
+
+  gtk_revealer_set_reveal_child (priv->notification, FALSE);
+}
+
+static gboolean
+on_remove_printer_timeout (gpointer user_data)
+{
+  on_notification_dismissed (NULL, user_data);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+on_printer_deleted (PpPrinterEntry *printer_entry,
+                    gpointer        user_data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+  GtkLabel               *label;
+  gchar                  *notification_message;
+  gchar                  *printer_name;
+
+  gtk_widget_hide (GTK_WIDGET (printer_entry));
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  on_notification_dismissed (NULL, self);
+
+  g_object_get (printer_entry,
+                "printer-name", &printer_name,
+                NULL);
+
+  /* Translators: %s is the printer name */
+  notification_message = g_strdup_printf (_("Printer “%s” has been deleted"),
+                                          printer_name);
+  label = (GtkLabel*)
+    gtk_builder_get_object (priv->builder, "notification-label");
+  gtk_label_set_label (label, notification_message);
+
+  g_free (notification_message);
+
+  priv->deleted_printer_name = g_strdup (printer_name);
+  g_free (printer_name);
+
+  gtk_revealer_set_reveal_child (priv->notification, TRUE);
+
+  priv->remove_printer_timeout_id = g_timeout_add_seconds (10, on_remove_printer_timeout, self);
+}
+
 static void
 on_printer_changed (PpPrinterEntry *printer_entry,
                     gpointer        user_data)
@@ -602,16 +744,27 @@ add_printer_entry (CcPrintersPanel *self,
   CcPrintersPanelPrivate *priv;
   PpPrinterEntry         *printer_entry;
   GtkWidget              *content;
+  GSList                 *widgets, *l;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
   content = (GtkWidget*) gtk_builder_get_object (priv->builder, "content");
 
   printer_entry = pp_printer_entry_new (printer, priv->is_authorized);
+
+  widgets = pp_printer_entry_get_size_group_widgets (printer_entry);
+  for (l = widgets; l != NULL; l = l->next)
+    gtk_size_group_add_widget (priv->size_group, GTK_WIDGET (l->data));
+  g_slist_free (widgets);
+
   g_signal_connect (printer_entry,
                     "printer-changed",
                     G_CALLBACK (on_printer_changed),
                     self);
+  g_signal_connect (printer_entry,
+                    "printer-delete",
+                    G_CALLBACK (on_printer_deleted),
+                    self);
 
   gtk_list_box_insert (GTK_LIST_BOX (content), GTK_WIDGET (printer_entry), -1);
   gtk_widget_show_all (content);
@@ -679,7 +832,12 @@ actualize_printers_list_cb (GObject      *source_object,
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "content");
   gtk_container_foreach (GTK_CONTAINER (widget), (GtkCallback) gtk_widget_destroy, NULL);
   for (i = 0; i < priv->num_dests; i++)
-    add_printer_entry (self, priv->dests[i]);
+    {
+      if (g_strcmp0 (priv->dests[i].name, priv->deleted_printer_name) == 0)
+          continue;
+
+      add_printer_entry (self, priv->dests[i]);
+    }
 }
 
 static void
@@ -714,7 +872,6 @@ new_printer_dialog_pre_response_cb (PpNewPrinterDialog *dialog,
   priv->new_printer_location = g_strdup (device_location);
   priv->new_printer_make_and_model = g_strdup (device_make_and_model);
   priv->new_printer_on_network = is_network_device;
-  priv->select_new_printer = TRUE;
 
   actualize_printers_list (self);
 }
@@ -726,13 +883,16 @@ new_printer_dialog_response_cb (PpNewPrinterDialog *dialog,
 {
   CcPrintersPanelPrivate *priv;
   CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+  GtkScrolledWindow      *scrolled_window;
+  GtkAllocation           allocation;
+  GtkAdjustment          *adjustment;
+  GtkWidget              *printer_entry;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
   if (priv->pp_new_printer_dialog)
     g_clear_object (&priv->pp_new_printer_dialog);
 
-  g_clear_pointer (&priv->new_printer_name, g_free);
   g_clear_pointer (&priv->new_printer_location, g_free);
   g_clear_pointer (&priv->new_printer_make_and_model, g_free);
 
@@ -754,6 +914,19 @@ new_printer_dialog_response_cb (PpNewPrinterDialog *dialog,
     }
 
   actualize_printers_list (self);
+
+  /* Scroll the view to show the newly added printer-entry. */
+  scrolled_window = GTK_SCROLLED_WINDOW (gtk_builder_get_object (priv->builder,
+                                                                 "scrolled-window"));
+  adjustment = gtk_scrolled_window_get_vadjustment (scrolled_window);
+
+  printer_entry = GTK_WIDGET (g_hash_table_lookup (priv->printer_entries,
+                                                   priv->new_printer_name));
+  gtk_widget_get_allocation (printer_entry, &allocation);
+  g_clear_pointer (&priv->new_printer_name, g_free);
+
+  gtk_adjustment_set_value (adjustment,
+                            allocation.y - gtk_widget_get_margin_top (printer_entry));
 }
 
 static void
@@ -993,7 +1166,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
   GtkWidget              *widget;
   PpCups                 *cups;
   GError                 *error = NULL;
-  gchar                  *objects[] = { "main-vbox", "headerbar-buttons", "search-button", NULL };
+  gchar                  *objects[] = { "overlay", "headerbar-buttons", "search-button", NULL };
   guint                   builder_result;
 
   priv = self->priv = PRINTERS_PANEL_PRIVATE (self);
@@ -1012,14 +1185,15 @@ cc_printers_panel_init (CcPrintersPanel *self)
   priv->cups_proxy = NULL;
   priv->cups_bus_connection = NULL;
   priv->dbus_subscription_id = 0;
+  priv->remove_printer_timeout_id = 0;
 
   priv->new_printer_name = NULL;
   priv->new_printer_location = NULL;
   priv->new_printer_make_and_model = NULL;
   priv->new_printer_on_network = FALSE;
-  priv->select_new_printer = FALSE;
 
   priv->renamed_printer_name = NULL;
+  priv->deleted_printer_name = NULL;
 
   priv->permission = NULL;
   priv->lockdown_settings = NULL;
@@ -1049,9 +1223,20 @@ cc_printers_panel_init (CcPrintersPanel *self)
     gtk_builder_get_object (priv->builder, "headerbar-buttons");
   priv->headerbar_buttons = widget;
 
+  priv->notification = (GtkRevealer*)
+    gtk_builder_get_object (priv->builder, "notification");
+
+  widget = (GtkWidget*)
+    gtk_builder_get_object (priv->builder, "notification-undo-button");
+  g_signal_connect (widget, "clicked", G_CALLBACK (on_printer_deletion_undone), self);
+
+  widget = (GtkWidget*)
+    gtk_builder_get_object (priv->builder, "notification-dismiss-button");
+  g_signal_connect (widget, "clicked", G_CALLBACK (on_notification_dismissed), self);
+
   /* add the top level widget */
   top_widget = (GtkWidget*)
-    gtk_builder_get_object (priv->builder, "main-vbox");
+    gtk_builder_get_object (priv->builder, "overlay");
 
   /* connect signals */
   widget = (GtkWidget*)
@@ -1100,6 +1285,8 @@ Please check your installation");
 
   priv->subscription_renew_cancellable = g_cancellable_new ();
 
+  priv->size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
   actualize_printers_list (self);
   attach_to_cups_notifier (self);
 
